package main

import (
	"io/ioutil"
	"os"
	"path"
	"path/filepath"
	"regexp"
	"sort"
	"strings"
	"text/template"
	"time"
)

type Dataset struct {
	Config          *Configuration
	Meta            string
	HasChangelog    bool
	Changelog       string
	HasPrivacy      bool
	Privacy         string
	HasMeta         bool
	Tasks           string
	HasTasks        bool
	Items           string
	HasItems        bool
	Requirements    string
	HasRequirements bool
	Created         time.Time
	CreatedFormat   string
}

func PrintOverview() error {

	err, pageData := collectStructureFromFiles(config.Path)
	if err != nil {
		return err
	}

	err, meta, hasMeta := buildRequirements(pageData)
	if err != nil {
		return err
	}

	err, changelog, hasChangelog := buildChangelog(pageData)
	if err != nil {
		return err
	}

	err, privacy, hasPrivacy := buildPrivacy(pageData)
	if err != nil {
		return err
	}

	err, tasks, hasTasks := buildTasksTable(pageData, false)
	if err != nil {
		return err
	}

	err, items, hasItems := buildItemOverviewTable(pageData, false)
	if err != nil {
		return err
	}

	t, err := template.New("overview").Parse(config.Overview.Template.Internal.MarkdownContent)
	if err != nil {
		return err
	}

	requirements := ""

	keys := make([]string, 0, len(pageData))
	for k := range pageData {
		keys = append(keys, k)
	}

	sort.Strings(keys)

	hasRequirements := false
	for _, k := range keys {
		hasRequirements = true
		p := pageData[k]
		requirements = requirements + getAdjustedContent(p.Absolute)
	}

	d := new(Dataset)

	d.Changelog = changelog
	d.HasChangelog = hasChangelog

	d.Privacy = privacy
	d.HasPrivacy = hasPrivacy

	d.Meta = meta
	d.HasMeta = hasMeta

	d.Tasks = tasks
	d.HasTasks = hasTasks

	d.Items = items
	d.HasItems = hasItems

	d.Requirements = requirements
	d.HasRequirements = hasRequirements

	d.Config = config
	d.Created = time.Now()
	d.CreatedFormat = time.Now().Format(config.Locale.DateFormat)
	//d.requirement = requirement

	outputName := arguments.Overview.Print.Output

	if arguments.Overview.Print.Format == "pdf" {

		if outputName == "" {
			printErrorAndExit(2, "if the type is pdf, the output option must be specified")
		}

		file, err := ioutil.TempFile(os.TempDir(), "reqman")
		if err != nil {
			printErrorAndExit(2, "A temporary file cannot be created", err.Error())
		}
		defer os.Remove(file.Name())

		err = t.Execute(file, d)
		if err != nil {
			return err
		}

		convertToPDF(file.Name(), outputName, config.Overview.Template.Latex)

	} else {

		if outputName != "" {
			file, err := os.Create(outputName)
			if err != nil {
				printErrorAndExit(2, "The output file cannot be written", err.Error())
			}

			// close fo on exit and check for its returned error
			defer func() {
				if err := file.Close(); err != nil {
					printErrorAndExit(2, "The output file cannot be written", err.Error())
				}
			}()

			err = t.Execute(file, d)
			if err != nil {
				return err
			}

		} else {

			err = t.Execute(os.Stdout, d)
			if err != nil {
				return err
			}
		}

	}

	return nil

}

func getAdjustedContent(absolute string) string {

	content, err := os.ReadFile(absolute)
	if err != nil {
		printError("The file cannot be read", absolute)
		return ""
	}

	err, def := splitYamlParts(content)
	if err != nil {
		printError(err.Error())
		return ""
	}

	s := convertImages(def.content, path.Dir(absolute))

	return s
}

func convertImages(content string, absolute string) string {

	todoRegEx := regexp.MustCompile(`(?P<match>\!\[(?P<label>[^]]*)\]\((?P<path>[^)]*)\))`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		content = strings.Replace(content, result["match"], "!["+result["label"]+"]("+path+")", -1)

	}

	return content
}

func convertTemplateLogo(content string, absolute string) string {
	todoRegEx := regexp.MustCompile(`(?m)^(?P<match>logo:\s*"?(?P<path>[^"\n]*)"?\s*)$`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		content = strings.Replace(content, result["match"], "logo: \""+path+"\"", -1)

	}

	return content
}

func convertTemplateLatexLogo(content string, absolute string) string {
	todoRegEx := regexp.MustCompile(`(?m)(?P<match>\\includegraphics[^{]*\{(?P<path>[^}]*)\})`)

	matches := todoRegEx.FindAllStringSubmatch(content, -1)
	if matches == nil {
		return content
	}

	for _, match := range matches {
		result := make(map[string]string)
		for i, name := range todoRegEx.SubexpNames() {
			if i != 0 && name != "" {
				result[name] = match[i]
			}
		}

		if filepath.IsAbs(result["path"]) {
			continue
		}

		path := path.Clean(absolute + "/" + result["path"])
		a := strings.Replace(result["match"], result["path"], path, -1)

		content = strings.Replace(content, result["match"], a, -1)

	}

	return content
}

func convertTemplateImages(content string, absolute string) string {
	content = convertTemplateLogo(content, absolute)
	content = convertTemplateLatexLogo(content, absolute)

	return content

}

func validateColumns(columns []string) []string {

	//for i, column := range columns {
	//	columns[i] = printer.Sprintf(column)
	//}

	// VALIDATE COLUMS NAME

	//
	//for _, name := range config.Requirements.Columns {
	//
	//	field := reflect.Indirect(r).FieldByName(name)
	//	if field == (reflect.Value{}) {
	//		printError("the specified column " + name + " for the table does not exist")
	//		continue

	//	}

	return columns

}
